@@ -35,7 +35,7 @@ http://tools.ietf.org/html/draft-pbryan-json-patch-04"""
 
 # Will be parsed by setup.py to determine package metadata
 __author__ = 'Stefan Kögl <stefan@skoegl.net>'
-__version__ = '0.3'
+__version__ = '0.4'
 __website__ = 'https://github.com/stefankoegl/python-json-patch'
 __license__ = 'Modified BSD License'
 
@@ -218,6 +218,8 @@ class JsonPatch(object):
         True
         """
         def compare_values(path, value, other):
+            if value == other:
+                return
             if isinstance(value, dict) and isinstance(other, dict):
                 for operation in compare_dict(path, value, other):
                     yield operation
@@ -231,25 +233,26 @@ class JsonPatch(object):
             for key in src:
                 if key not in dst:
                     yield {'remove': '/'.join(path + [key])}
-                elif src[key] != dst[key]:
-                    current = path + [key]
-                    for operation in compare_values(current, src[key], dst[key]):
-                        yield operation
+                    continue
+                current = path + [key]
+                for operation in compare_values(current, src[key], dst[key]):
+                    yield operation
             for key in dst:
                 if key not in src:
                     yield {'add': '/'.join(path + [key]), 'value': dst[key]}
 
         def compare_list(path, src, dst):
             lsrc, ldst = len(src), len(dst)
-            for idx in reversed(range(max(lsrc, ldst))):
-                if idx < lsrc and idx < ldst:
+            for idx in range(min(lsrc, ldst)):
+                current = path + [str(idx)]
+                for operation in compare_values(current, src[idx], dst[idx]):
+                    yield operation
+            if lsrc < ldst:
+                for idx in range(lsrc, ldst):
                     current = path + [str(idx)]
-                    for operation in compare_values(current, src[idx], dst[idx]):
-                        yield operation
-                elif idx < ldst:
-                    yield {'add': '/'.join(path + [str(idx)]),
-                           'value': dst[idx]}
-                elif idx < lsrc:
+                    yield {'add': '/'.join(current), 'value': dst[idx]}
+            elif lsrc > ldst:
+                for idx in reversed(range(ldst, lsrc)):
                     yield {'remove': '/'.join(path + [str(idx)])}
 
         return cls(list(compare_dict([''], src, dst)))
